#!/usr/bin/python
#
# Copyright (c) Michael Tharp <gxti@partiallystapled.com>
#
# This file is distributed under the terms of the MIT License.
# See the LICENSE file at the top of this tree, or if it is missing a copy can
# be found at http://opensource.org/licenses/MIT
#

"""
Take a SVG file with a single template label, step it on a grid, fill in serial
numbers from a CSV, and output a new SVG file for printing.
"""

import copy
import csv
import sys
from xml.etree import ElementTree

SUB_LAYER = 'layer1'
SUB_MAC = '01:23:45:AB:CD:EF'
SUB_SER = '#9999999'
X_COUNT = 3
X_STEP = 199.53
Y_COUNT = 5
Y_STEP = 158.22

NS_SVG = '{http://www.w3.org/2000/svg}'
NS_SP = '{http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd}'
NS_IS = '{http://www.inkscape.org/namespaces/inkscape}'


def main():
    inpath, outpath, csvpath, first_serial, first_label = sys.argv[1:]
    first_label = int(first_label)
    ElementTree.register_namespace('dc', 'http://purl.org/dc/elements/1.1/')
    ElementTree.register_namespace('cc', 'http://creativecommons.org/ns#')
    ElementTree.register_namespace('rdf', 'http://www.w3.org/1999/02/22-rdf-syntax-ns#')
    ElementTree.register_namespace('sodipodi', NS_SP[1:-1])
    ElementTree.register_namespace('inkscape', NS_IS[1:-1])
    ElementTree.register_namespace('', NS_SVG[1:-1])
    cr = csv.DictReader(open(csvpath))
    for next_serial in cr:
        if next_serial['Serial'] == first_serial:
            break
    else:
        sys.exit("first serial '%s' not found" % first_serial)
    t = ElementTree.parse(open(inpath))
    for nv in t.iter(NS_SP + 'namedview'):
        nv.attrib['showgrid'] = 'false'
        nv.attrib['showguides'] = 'false'
    for group in t.iter(NS_SVG + 'g'):
        if group.attrib.get(NS_IS + 'label') == 'ol8325':
            group.attrib['style'] = 'display:none'
    for group in t.iter(NS_SVG + 'g'):
        if group.attrib['id'] == 'layer1':
            break
    else:
        raise RuntimeError("missing layer1")
    stuff = group[:]
    group[:] = []
    for elem in stuff:
        for child in elem.iter():
            if 'id' in elem.attrib:
                del elem.attrib['id']
    for y in range(Y_COUNT):
        for x in range(X_COUNT):
            if first_label:
                first_label -= 1
                continue
            if next_serial:
                block = ElementTree.SubElement(group, NS_SVG + 'g')
                block.attrib['transform'] = 'translate(%s,%s)' % (x * X_STEP, y * Y_STEP)
                block[:] = copy.deepcopy(stuff)
                for span in block.iter(NS_SVG + 'tspan'):
                    if not span.text:
                        continue
                    span.text = span.text.replace(SUB_MAC, next_serial['MAC']
                            ).replace(SUB_SER, '#' + next_serial['Serial'])
                try:
                    next_serial = cr.next()
                except StopIteration:
                    next_serial = None
                    break
        if not next_serial:
            break
    with open(outpath, 'w') as f:
        t.write(f)


main()
